@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2015 David Robillard <http://drobilla.net>
+  Copyright 2007-2017 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -23,6 +23,7 @@
 #include "./suil_internal.h"
 
 #define GTK2_UI_URI  LV2_UI__GtkUI
+#define GTK3_UI_URI  LV2_UI__Gtk3UI
 #define QT4_UI_URI   LV2_UI__Qt4UI
 #define QT5_UI_URI   LV2_UI__Qt5UI
 #define X11_UI_URI   LV2_UI__X11UI
@@ -31,7 +32,7 @@
 
 SUIL_API
 unsigned
-suil_ui_supported(const char* container_type_uri,
+suil_ui_supported(const char* host_type_uri,
                   const char* ui_type_uri)
 {
 	enum {
@@ -39,23 +40,27 @@ suil_ui_supported(const char* container_type_uri,
 		SUIL_WRAPPING_NATIVE      = 1,
 		SUIL_WRAPPING_EMBEDDED    = 2
 	};
-	if (!strcmp(container_type_uri, ui_type_uri)) {
+	if (!strcmp(host_type_uri, ui_type_uri)) {
 		return SUIL_WRAPPING_NATIVE;
-	} else if ((!strcmp(container_type_uri, GTK2_UI_URI)
+	} else if ((!strcmp(host_type_uri, GTK2_UI_URI)
 	            && !strcmp(ui_type_uri, QT4_UI_URI))
-	           || (!strcmp(container_type_uri, QT4_UI_URI)
+	           || (!strcmp(host_type_uri, GTK2_UI_URI)
+	               && !strcmp(ui_type_uri, QT5_UI_URI))
+	           || (!strcmp(host_type_uri, QT4_UI_URI)
 	               && !strcmp(ui_type_uri, GTK2_UI_URI))
-	           || (!strcmp(container_type_uri, QT5_UI_URI)
+	           || (!strcmp(host_type_uri, QT5_UI_URI)
 	               && !strcmp(ui_type_uri, GTK2_UI_URI))
-	           || (!strcmp(container_type_uri, GTK2_UI_URI)
+	           || (!strcmp(host_type_uri, GTK2_UI_URI)
 	               && !strcmp(ui_type_uri, X11_UI_URI))
-	           || (!strcmp(container_type_uri, GTK2_UI_URI)
+	           || (!strcmp(host_type_uri, GTK3_UI_URI)
+	               && !strcmp(ui_type_uri, X11_UI_URI))
+	           || (!strcmp(host_type_uri, GTK2_UI_URI)
 	               && !strcmp(ui_type_uri, WIN_UI_URI))
-	           || (!strcmp(container_type_uri, GTK2_UI_URI)
+	           || (!strcmp(host_type_uri, GTK2_UI_URI)
 	               && !strcmp(ui_type_uri, COCOA_UI_URI))
-	           || (!strcmp(container_type_uri, QT4_UI_URI)
+	           || (!strcmp(host_type_uri, QT4_UI_URI)
 	               && !strcmp(ui_type_uri, X11_UI_URI))
-	           || (!strcmp(container_type_uri, QT5_UI_URI)
+	           || (!strcmp(host_type_uri, QT5_UI_URI)
 	               && !strcmp(ui_type_uri, X11_UI_URI))) {
 		return SUIL_WRAPPING_EMBEDDED;
 	} else {
@@ -89,12 +94,24 @@ open_wrapper(SuilHost*      host,
 		module_name = "suil_qt4_in_gtk2";
 	}
 #endif
+#ifdef SUIL_WITH_QT5_IN_GTK2
+	if (!strcmp(container_type_uri, GTK2_UI_URI)
+	    && !strcmp(ui_type_uri, QT5_UI_URI)) {
+		module_name = "suil_qt5_in_gtk2";
+	}
+#endif
 #ifdef SUIL_WITH_X11_IN_GTK2
 	if (!strcmp(container_type_uri, GTK2_UI_URI)
 	    && !strcmp(ui_type_uri, X11_UI_URI)) {
 		module_name = "suil_x11_in_gtk2";
 	}
 #endif
+#ifdef SUIL_WITH_X11_IN_GTK3
+	if (!strcmp(container_type_uri, GTK3_UI_URI)
+	    && !strcmp(ui_type_uri, X11_UI_URI)) {
+		module_name = "suil_x11_in_gtk3";
+	}
+#endif
 #ifdef SUIL_WITH_WIN_IN_GTK2
 	if (!strcmp(container_type_uri, GTK2_UI_URI)
 	    && !strcmp(ui_type_uri, WIN_UI_URI)) {
@@ -126,25 +143,8 @@ open_wrapper(SuilHost*      host,
 		return NULL;
 	}
 
-	const char* const env_dir = getenv("SUIL_MODULE_DIR");
-	const char* const mod_dir = env_dir ? env_dir : SUIL_MODULE_DIR;
-
-	const size_t path_len = strlen(mod_dir)
-		+ strlen(SUIL_DIR_SEP SUIL_MODULE_PREFIX SUIL_MODULE_EXT)
-		+ strlen(module_name)
-		+ 2;
-
-	char* const path = (char*)calloc(path_len, 1);
-	snprintf(path, path_len, "%s%s%s%s%s",
-	         mod_dir, SUIL_DIR_SEP,
-	         SUIL_MODULE_PREFIX, module_name, SUIL_MODULE_EXT);
-
-	// Open wrap module
-	dlerror();
-	void* lib = dlopen(path, RTLD_NOW);
+	void* const lib = suil_open_module(module_name);
 	if (!lib) {
-		SUIL_ERRORF("Unable to open wrap module %s (%s)\n", path, dlerror());
-		free(path);
 		return NULL;
 	}
 
@@ -159,17 +159,13 @@ open_wrapper(SuilHost*      host,
 		              n_features)
 		: NULL;
 
-	if (!wrapper) {
-		SUIL_ERRORF("Corrupt module %s\n", path);
+	if (wrapper) {
+		wrapper->lib = lib;
+	} else {
+		SUIL_ERRORF("Corrupt wrap module %s\n", module_name);
 		dlclose(lib);
-		free(path);
-		return NULL;
 	}
 
-	free(path);
-
-	wrapper->lib = lib;
-
 	return wrapper;
 }
 
