@@ -14,16 +14,15 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-#include <string.h>
-
-#include <gtk/gtk.h>
 #include <gdk/gdkx.h>
-
-#include "./suil_internal.h"
+#include <gtk/gtk.h>
+#include <string.h>
 
 #include "lv2/lv2plug.in/ns/ext/options/options.h"
 #include "lv2/lv2plug.in/ns/ext/urid/urid.h"
 
+#include "./suil_internal.h"
+
 #define SUIL_TYPE_X11_WRAPPER (suil_x11_wrapper_get_type())
 #define SUIL_X11_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), SUIL_TYPE_X11_WRAPPER, SuilX11Wrapper))
 
@@ -69,9 +68,11 @@ x_window_is_valid(SuilX11Wrapper* socket)
 	           &root, &parent, &children, &childcount);
 	for (unsigned i = 0; i < childcount; ++i) {
 		if (children[i] == (Window)socket->instance->ui_widget) {
+			XFree(children);
 			return true;
 		}
 	}
+	XFree(children);
 	return false;
 }
 
@@ -192,6 +193,7 @@ forward_size_request(SuilX11Wrapper* socket,
 		int        width  = allocation->width;
 		int        height = allocation->height;
 		XSizeHints hints;
+		memset(&hints, 0, sizeof(hints));
 		XGetNormalHints(GDK_WINDOW_XDISPLAY(window),
 		                (Window)socket->instance->ui_widget,
 		                &hints);
@@ -266,10 +268,11 @@ suil_x11_wrapper_class_init(SuilX11WrapperClass* klass)
 	GObjectClass* const   gobject_class = G_OBJECT_CLASS(klass);
 	GtkWidgetClass* const widget_class  = GTK_WIDGET_CLASS(klass);
 
-	gobject_class->finalize       = suil_x11_wrapper_finalize;
-	widget_class->realize         = suil_x11_wrapper_realize;
-	widget_class->show            = suil_x11_wrapper_show;
-	widget_class->key_press_event = suil_x11_wrapper_key_event;
+	gobject_class->finalize         = suil_x11_wrapper_finalize;
+	widget_class->realize           = suil_x11_wrapper_realize;
+	widget_class->show              = suil_x11_wrapper_show;
+	widget_class->key_press_event   = suil_x11_wrapper_key_event;
+	widget_class->key_release_event = suil_x11_wrapper_key_event;
 }
 
 static void
